
#include <OPENR/OSyslog.h>
#include <OPENR/OPENRAPI.h>
#include <ant.h>
#include <EndpointTypes.h>
#include <TCPEndpointMsg.h>
#include <UDPEndpointMsg.h>
#include <string.h>
#include "Wireless.h"
#include "Socket.h"
#include "MMCombo/entry.h"

//TODO: Support listen backlogs
//TODO: Implement singleton

using namespace SocketNS;

Wireless *wireless=NULL;

Wireless::Wireless ()
  : ipstackRef(), myOID(), sock_num(0)
{
  ipstackRef = antStackRef("IPStack");
  WhoAmI(&myOID);

  for (int sock = 0; sock < WIRELESS_MAX_SOCKETS; sock++) {
   sockets[sock]=NULL;
  }
}    

Wireless::~Wireless ()
{
  // TODO
}

Socket* Wireless::socket(TransportType_t ttype)
{
  return socket(ttype, WIRELESS_DEF_RECV_SIZE, WIRELESS_DEF_SEND_SIZE);
}

Socket* Wireless::socket(TransportType_t ttype, int recvsize, int sendsize)
{
  if (sock_num >= WIRELESS_MAX_SOCKETS
      || (recvsize + sendsize) <= 256) return NULL;

  sockets[sock_num]=new Socket(sock_num);
  sockets[sock_num]->trType=ttype;
  sockets[sock_num]->sendBufSize=sendsize;
  sockets[sock_num]->recvBufSize=recvsize;

  // setup send buffer
  antEnvCreateSharedBufferMsg sendBufferMsg(sendsize*2);
  sendBufferMsg.Call(ipstackRef, sizeof(sendBufferMsg));
  if (sendBufferMsg.error != ANT_SUCCESS) return NULL;
  
  sockets[sock_num]->sendBuffer = sendBufferMsg.buffer;
  sockets[sock_num]->sendBuffer.Map();
  sockets[sock_num]->sendData
        = (byte*)(sockets[sock_num]->sendBuffer.GetAddress());

  // setup receive buffer
  antEnvCreateSharedBufferMsg recvBufferMsg(recvsize*2);
  recvBufferMsg.Call(ipstackRef, sizeof(recvBufferMsg));
  if (recvBufferMsg.error != ANT_SUCCESS) return NULL;
  
  sockets[sock_num]->recvBuffer = recvBufferMsg.buffer;
  sockets[sock_num]->recvBuffer.Map();
  sockets[sock_num]->recvData
        = (byte*)(sockets[sock_num]->recvBuffer.GetAddress());

  sockets[sock_num]->readData=sockets[sock_num]->recvData+recvsize;
  sockets[sock_num]->writeData=sockets[sock_num]->sendData+sendsize;

  return sockets[sock_num++]; 
}

int Wireless::listen(int sock, int port)
{
  if (port<=0 || port>=65535
      || sock<0 || sock>=sock_num
      || sockets[sock]->state != CONNECTION_CLOSED) return -1;
  
  sockets[sock]->server_port=port;
  sockets[sock]->init();

  if (sockets[sock]->trType==SOCK_STREAM) {
    // create endpoint
    antEnvCreateEndpointMsg tcpCreateMsg(EndpointType_TCP,
                                         (sockets[sock]->recvBufSize+
                                         sockets[sock]->sendBufSize)*3);
    tcpCreateMsg.Call(ipstackRef, sizeof(tcpCreateMsg));
    if (tcpCreateMsg.error != ANT_SUCCESS) return -1;
    sockets[sock]->endpoint = tcpCreateMsg.moduleRef;

    // listen
    TCPEndpointListenMsg listenMsg(sockets[sock]->endpoint,
                                   IP_ADDR_ANY, port);
    listenMsg.continuation = (void*)sock;

    listenMsg.Send(ipstackRef, myOID,
                   Extra_Entry[entryListenCont], sizeof(listenMsg));
    
    sockets[sock]->state = CONNECTION_LISTENING;
    return 0;
  } else if (sockets[sock]->trType==SOCK_DGRAM) {
    // create endpoint
    antEnvCreateEndpointMsg udpCreateMsg(EndpointType_UDP,
                                         (sockets[sock]->recvBufSize+
                                         sockets[sock]->sendBufSize)*3);
    udpCreateMsg.Call(ipstackRef, sizeof(udpCreateMsg));
    if (udpCreateMsg.error != ANT_SUCCESS) return -1;
    sockets[sock]->endpoint = udpCreateMsg.moduleRef;

    // bind
    UDPEndpointBindMsg bindMsg(sockets[sock]->endpoint,
                                   IP_ADDR_ANY, port);
    bindMsg.continuation = (void*)sock;

    bindMsg.Send(ipstackRef, myOID,
                 Extra_Entry[entryBindCont], sizeof(bindMsg));
    
    sockets[sock]->state = CONNECTION_LISTENING;
    return 0;
  } else {
    return -1;
  }
}

int Wireless::connect(int sock, const char* ipaddr, int port)
{
  if (port<=0 || port>=65535
      || sock<0 || sock>=sock_num
      || sockets[sock]->state != CONNECTION_CLOSED) return -1;
  
  sockets[sock]->init();
  if (sockets[sock]->trType==SOCK_STREAM) {
    // create endpoint
    antEnvCreateEndpointMsg tcpCreateMsg(EndpointType_TCP,
                                         (sockets[sock]->recvBufSize+
                                         sockets[sock]->sendBufSize)*3);
    tcpCreateMsg.Call(ipstackRef, sizeof(tcpCreateMsg));
    if (tcpCreateMsg.error != ANT_SUCCESS) return -1;
    sockets[sock]->endpoint = tcpCreateMsg.moduleRef;

    // connect
    TCPEndpointConnectMsg connectMsg(sockets[sock]->endpoint,
                                    IP_ADDR_ANY, IP_PORT_ANY, ipaddr, port);
    connectMsg.continuation = (void*)sock;

    connectMsg.Send(ipstackRef, myOID,
                    Extra_Entry[entryConnectCont], sizeof(connectMsg));
    
    sockets[sock]->state = CONNECTION_CONNECTING;
    return 0;
  } else if (sockets[sock]->trType==SOCK_DGRAM) {
    //TODO
    return 0;
  } else {
    return -1;
  }
}

void
Wireless::ListenCont(void* msg)
{
  TCPEndpointListenMsg* listenMsg = (TCPEndpointListenMsg*)msg;
  int sock = (int)listenMsg->continuation;

  if (listenMsg->error != TCP_SUCCESS) {
    sockets[sock]->state = CONNECTION_ERROR;
    // no use recycling since its a resource issue
    return;
  }

  sockets[sock]->state = CONNECTION_CONNECTED;
  if (sockets[sock]->rcvcbckfn) { receive(sock); }
}

void
Wireless::ConnectCont(void *msg)
{
  TCPEndpointConnectMsg* connectMsg = (TCPEndpointConnectMsg*)msg;
  int sock = (int)connectMsg->continuation;

  if (connectMsg->error != TCP_SUCCESS) {
    sockets[sock]->state = CONNECTION_ERROR;
    return;
  }
  
  sockets[sock]->state = CONNECTION_CONNECTED;
}

void
Wireless::BindCont(void *msg)
{
  UDPEndpointBindMsg* bindMsg = (UDPEndpointBindMsg*)msg;
  int sock = (int)bindMsg->continuation;

  if (bindMsg->error != UDP_SUCCESS) {
    sockets[sock]->state = CONNECTION_ERROR;
    return;
  }

  sockets[sock]->state = CONNECTION_CONNECTED;
}

void
Wireless::send(int sock)
{
  if (sock<0 || sock>=sock_num
      || sockets[sock]->state != CONNECTION_CONNECTED
      || sockets[sock]->sendSize <= 0) return;

    TCPEndpointSendMsg sendMsg(sockets[sock]->endpoint,
                               sockets[sock]->sendData,
                               sockets[sock]->sendSize);
    sendMsg.continuation = (void*)sock;

    sockets[sock]->tx=true;
    sendMsg.Send(ipstackRef, myOID,
                 Extra_Entry[entrySendCont],
                 sizeof(TCPEndpointSendMsg));
    sockets[sock]->sendSize = 0;
}

void
Wireless::SendCont(void* msg)
{
    TCPEndpointSendMsg* sendMsg = (TCPEndpointSendMsg*)msg;
    int sock = (int)(sendMsg->continuation);

    sockets[sock]->tx=false;
    if (sendMsg->error != TCP_SUCCESS) {
      sockets[sock]->state=CONNECTION_ERROR;
      close(sock);
      return;
    }
    sockets[sock]->flush();
}

void
Wireless::blockingSend(int sock)
{
  if (sock<0 || sock>=sock_num
      || sockets[sock]->state != CONNECTION_CONNECTED
      || sockets[sock]->sendSize <= 0) return;

    TCPEndpointSendMsg sendMsg(sockets[sock]->endpoint,
                               sockets[sock]->sendData,
                               sockets[sock]->sendSize);
    sendMsg.continuation = (void*)sock;

    sockets[sock]->tx=true;
    sockets[sock]->sendSize = 0;
    sendMsg.Call(ipstackRef, sizeof(TCPEndpointSendMsg));
    sockets[sock]->tx=false;
    // no double buffering
}

void
Wireless::setReceiver(int sock, int (*rcvcbckfn) (char*, int) )
{
  if (sock<0 || sock>=sock_num) return;

  sockets[sock]->rcvcbckfn=rcvcbckfn;
}

void
Wireless::receive(int sock)
{
  if (sock<0 || sock>=sock_num
      || sockets[sock]->state != CONNECTION_CONNECTED) return;
  
  TCPEndpointReceiveMsg receiveMsg(sockets[sock]->endpoint,
                                   sockets[sock]->recvData,
                                   1,sockets[sock]->recvBufSize);
  receiveMsg.continuation = (void*)sock;

  receiveMsg.Send(ipstackRef, myOID,
                  Extra_Entry[entryReceiveCont], sizeof(receiveMsg));

  sockets[sock]->rx=true;
}

void
Wireless::receive(int sock, int (*rcvcbckfn) (char*, int) )
{
  if (sock<0 || sock>=sock_num
      || sockets[sock]->state != CONNECTION_CONNECTED) return;

  sockets[sock]->rcvcbckfn=rcvcbckfn;
  
  TCPEndpointReceiveMsg receiveMsg(sockets[sock]->endpoint,
                                   sockets[sock]->recvData,
                                   1,sockets[sock]->recvBufSize);
  receiveMsg.continuation = (void*)sock;

  receiveMsg.Send(ipstackRef, myOID,
                  Extra_Entry[entryReceiveCont], sizeof(receiveMsg));
  sockets[sock]->rx=true;
}

void
Wireless::ReceiveCont(void* msg)
{
    TCPEndpointReceiveMsg* receiveMsg = (TCPEndpointReceiveMsg*)msg;
    int sock = (int)(receiveMsg->continuation);

    if (receiveMsg->error != TCP_SUCCESS) {
      sockets[sock]->state=CONNECTION_ERROR;
      close(sock);
      return;
    }
    sockets[sock]->recvSize=receiveMsg->sizeMin;
    sockets[sock]->rcvcbckfn((char*)sockets[sock]->recvData,
                             sockets[sock]->recvSize);
    sockets[sock]->recvSize=0;
    receive(sock);
}

void
Wireless::close(int sock)
{
  if (sockets[sock]->state == CONNECTION_CLOSED ||
      sockets[sock]->state == CONNECTION_CLOSING) return;

  TCPEndpointCloseMsg closeMsg(sockets[sock]->endpoint);
  closeMsg.continuation = (void*)sock;

  closeMsg.Send(ipstackRef, myOID,
      Extra_Entry[entryCloseCont], sizeof(closeMsg));

  sockets[sock]->state = CONNECTION_CLOSING;
}

void
Wireless::close(Socket &sobj)
{
  close(sobj.sock);
}

void
Wireless::close(Socket *sobj)
{
  close(sobj->sock);
}

void
Wireless::CloseCont(void* msg)
{
  TCPEndpointCloseMsg* closeMsg = (TCPEndpointCloseMsg*)msg;
  int sock = (int)(closeMsg->continuation);

  sockets[sock]->state = CONNECTION_CLOSED;
  if (sockets[sock]->server_port>0) {
    // recycle if server
    if (sockets[sock]->server_port>0)
      listen(sock, sockets[sock]->server_port);
  }
}

